/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.pcDevice.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.device.db.entity.CdEngineInfo;
import net.northking.cctp.device.db.entity.CdPcDevice;
import net.northking.cctp.device.db.entity.CdPcUser;
import net.northking.cctp.device.dto.pcDevice.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * PC设备管理Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CdPcDeviceApiService extends ExcelService
{
  /**
   * 查询PC设备管理列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CdPcDeviceDetailDto> pagingQuery(QueryByPage<CdPcDeviceQueryDto> queryByPage);

  /**
   * 新增PC设备管理
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  Map<String,String> add(CdPcDeviceAddDto dto, HttpServletRequest request);

  /**
   * 更新PC设备管理
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CdPcDeviceUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取PC设备管理
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CdPcDeviceDetailDto findByPK(String uuid);

  /**
   * 删除PC设备管理
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);


    Pagination<CdPcDeviceDetailDto> pagingQueryByTenant(QueryByPage<CdPcDeviceQueryDto> queryByPage);

    Pagination<CdPcDeviceDetailDto> pagingNoneTenant(QueryByPage<CdPcDeviceQueryDto> queryByPage);

    Pagination<CdPcDeviceDetailDto> pagingByTenant(QueryByPage<CdPcDeviceQueryDto> queryByPage);

    Pagination<CdPcDeviceDetailDto> pagingNoneProject(QueryByPage<CdPcDeviceQueryDto> queryByPage);

    Pagination<CdPcDeviceDetailDto> pagingByProject(QueryByPage<CdPcDeviceQueryDto> queryByPage);

    String registerPC(CdPcDevice cdPcDevice);

    CdEngineInfo findEngineById(String id);

    CdPcDeviceRemoteDto achieve(CdDeviceRemoteDto dto, HttpServletRequest request);

    Pagination<ScriptPcResultDto> scriptDeviceByPro(QueryByPage<CdPcDeviceScriptDto> queryByPage);

    List<CdPcUser> queryDeviceUser(String deviceId);

    Integer updatePcUser(CdPcUserUpdateDto dto);
}
