/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import net.northking.cctp.testbug.db.entity.BugManagement;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.List;

/**
* 查询接口--数据传输对象：缺陷管理
*
* createdate:  2021-10-08 16:05:30  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@ApiModel(description = "缺陷管理")
public class BugManagementQueryDto extends BugManagement implements Serializable
{
    /**
     * 严重程度（建议性错误-E类、警告性错误-D类、一般性错误-C类、严重性错误-B类、致命性错误-A类）
     <br>
     */
    @ApiModelProperty("严重程度（建议性错误-E类、警告性错误-D类、一般性错误-C类、严重性错误-B类、致命性错误-A类） ")
    private List<String> bugGrades;
    /**
     * 缺陷分类(1.需求缺陷、2.设计缺陷、3.编码缺陷、4.UI缺陷、5.兼容性缺陷、6.环境缺陷、7.用户体验、8移植缺陷)<br>
     */
    @ApiModelProperty("缺陷分类(1.需求缺陷、2.设计缺陷、3.编码缺陷、4.UI缺陷、5.兼容性缺陷、6.环境缺陷、7.用户体验、8移植缺陷)")
    private List<String> bugTypes;
    /**
     * 紧急程度：1.高、2.中、3.低<br>
     */
    @ApiModelProperty("紧急程度：1.高、2.中、3.低")
    private List<String> emergencies;
    /**
     * 缺陷状态：1.新建、2.打开、3.已分配、4.拒绝、5.待验证、6.处理中、7.延迟、8.已修复、9.关闭、10.无效、11.重新打开、12.挂起、13.待裁决、14.待完善、15.待分配、16.待修复、17.待审核<br>
     */
    @ApiModelProperty("缺陷状态：1.新建、2.打开、3.已分配、4.拒绝、5.待验证、6.处理中、7.延迟、8.已修复、9.关闭、10.无效、11.重新打开、12.挂起、13.待裁决、14.待完善、15.待分配、16.待修复、17.待审核")
    private List<String> states;
    /**
     * 创建发现人<br>
     */
    @ApiModelProperty("创建发现人")
    private List<String> createdBys;
    /**
     * 创建时间开始<br>
     */
    @ApiModelProperty("创建时间开始")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String createdTimeStart;
    /**
     * 创建时间结束<br>
     */
    @ApiModelProperty("创建时间结束")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String createdTimeEnd;
    /**
     * 分配给，缺陷接收人<br>
     */
    @ApiModelProperty("分配给，缺陷接收人")
    private List<String> receivers;
    /**
     * 修复时间开始<br>
     */
    @ApiModelProperty("修复时间开始")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String repairTimeStart;
    /**
     * 修复时间关闭<br>
     */
    @ApiModelProperty("修复时间关闭")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String repairTimeEnd;
    /**
     * 修复时间<br>
     */
    @ApiModelProperty("修复人")
    private List<String> repairUsers;
    /**
     * 关闭日期开始<br>
     */
    @ApiModelProperty("关闭日期开始")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String closeTimeStart;
    /**
     * 关闭日期结束<br>
     */
    @ApiModelProperty("关闭日期结束")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String closeTimeEnd;

    @ApiModelProperty("缺陷id")
    private List<String> bugIds;

    @ApiModelProperty("项目id")
    private List<String> projectList;

    @ApiModelProperty("已删除的项目id")
    private List<String> deletedProjectList;

    @ApiModelProperty("USER_DEFINED_IDX1")
    @JsonProperty("")
    private String USER_DEFINED_IDX1;

    @ApiModelProperty("USER_DEFINED_IDX2")
    @JsonProperty("")
    private String USER_DEFINED_IDX2;

    @ApiModelProperty("USER_DEFINED_IDX3")
    @JsonProperty("")
    private String USER_DEFINED_IDX3;

    @ApiModelProperty("USER_DEFINED_IDX4")
    @JsonProperty("")
    private String USER_DEFINED_IDX4;

    @ApiModelProperty("USER_DEFINED_IDX5")
    @JsonProperty("")
    private String USER_DEFINED_IDX5;

    @ApiModelProperty("USER_DEFINED_IDX6")
    @JsonProperty("")
    private String USER_DEFINED_IDX6;

    @ApiModelProperty("USER_DEFINED_IDX7")
    @JsonProperty("")
    private String USER_DEFINED_IDX7;

    @ApiModelProperty("USER_DEFINED_IDX8")
    @JsonProperty("")
    private String USER_DEFINED_IDX8;

    @ApiModelProperty("USER_DEFINED_IDX9")
    @JsonProperty("")
    private String USER_DEFINED_IDX9;

    @ApiModelProperty("USER_DEFINED_IDX10")
    @JsonProperty("")
    private String USER_DEFINED_IDX10;

    @ApiModelProperty("USER_DEFINED_IDX11")
    @JsonProperty("")
    private String USER_DEFINED_IDX11;

    @ApiModelProperty("USER_DEFINED_IDX12")
    @JsonProperty("")
    private String USER_DEFINED_IDX12;

    @ApiModelProperty("USER_DEFINED_IDX13")
    @JsonProperty("")
    private String USER_DEFINED_IDX13;

    @ApiModelProperty("USER_DEFINED_IDX14")
    @JsonProperty("")
    private String USER_DEFINED_IDX14;

    @ApiModelProperty("USER_DEFINED_IDX15")
    @JsonProperty("")
    private String USER_DEFINED_IDX15;

    @ApiModelProperty("USER_DEFINED_IDX16")
    @JsonProperty("")
    private String USER_DEFINED_IDX16;

    @ApiModelProperty("USER_DEFINED_IDX17")
    @JsonProperty("")
    private String USER_DEFINED_IDX17;

    @ApiModelProperty("USER_DEFINED_IDX18")
    @JsonProperty("")
    private String USER_DEFINED_IDX18;

    @ApiModelProperty("USER_DEFINED_IDX19")
    @JsonProperty("")
    private String USER_DEFINED_IDX19;

    @ApiModelProperty("USER_DEFINED_IDX20")
    @JsonProperty("")
    private String USER_DEFINED_IDX20;

    @ApiModelProperty("关联id")
    private String linkId;

    public String getLinkId() {
        return linkId;
    }

    public void setLinkId(String linkId) {
        this.linkId = linkId;
    }

    public String getUSER_DEFINED_IDX1() {
        return USER_DEFINED_IDX1;
    }

    public void setUSER_DEFINED_IDX1(String USER_DEFINED_IDX1) {
        this.USER_DEFINED_IDX1 = USER_DEFINED_IDX1;
    }

    public String getUSER_DEFINED_IDX2() {
        return USER_DEFINED_IDX2;
    }

    public void setUSER_DEFINED_IDX2(String USER_DEFINED_IDX2) {
        this.USER_DEFINED_IDX2 = USER_DEFINED_IDX2;
    }

    public String getUSER_DEFINED_IDX3() {
        return USER_DEFINED_IDX3;
    }

    public void setUSER_DEFINED_IDX3(String USER_DEFINED_IDX3) {
        this.USER_DEFINED_IDX3 = USER_DEFINED_IDX3;
    }

    public String getUSER_DEFINED_IDX4() {
        return USER_DEFINED_IDX4;
    }

    public void setUSER_DEFINED_IDX4(String USER_DEFINED_IDX4) {
        this.USER_DEFINED_IDX4 = USER_DEFINED_IDX4;
    }

    public String getUSER_DEFINED_IDX5() {
        return USER_DEFINED_IDX5;
    }

    public void setUSER_DEFINED_IDX5(String USER_DEFINED_IDX5) {
        this.USER_DEFINED_IDX5 = USER_DEFINED_IDX5;
    }

    public String getUSER_DEFINED_IDX6() {
        return USER_DEFINED_IDX6;
    }

    public void setUSER_DEFINED_IDX6(String USER_DEFINED_IDX6) {
        this.USER_DEFINED_IDX6 = USER_DEFINED_IDX6;
    }

    public String getUSER_DEFINED_IDX7() {
        return USER_DEFINED_IDX7;
    }

    public void setUSER_DEFINED_IDX7(String USER_DEFINED_IDX7) {
        this.USER_DEFINED_IDX7 = USER_DEFINED_IDX7;
    }

    public String getUSER_DEFINED_IDX8() {
        return USER_DEFINED_IDX8;
    }

    public void setUSER_DEFINED_IDX8(String USER_DEFINED_IDX8) {
        this.USER_DEFINED_IDX8 = USER_DEFINED_IDX8;
    }

    public String getUSER_DEFINED_IDX9() {
        return USER_DEFINED_IDX9;
    }

    public void setUSER_DEFINED_IDX9(String USER_DEFINED_IDX9) {
        this.USER_DEFINED_IDX9 = USER_DEFINED_IDX9;
    }

    public String getUSER_DEFINED_IDX10() {
        return USER_DEFINED_IDX10;
    }

    public void setUSER_DEFINED_IDX10(String USER_DEFINED_IDX10) {
        this.USER_DEFINED_IDX10 = USER_DEFINED_IDX10;
    }

    public String getUSER_DEFINED_IDX11() {
        return USER_DEFINED_IDX11;
    }

    public void setUSER_DEFINED_IDX11(String USER_DEFINED_IDX11) {
        this.USER_DEFINED_IDX11 = USER_DEFINED_IDX11;
    }

    public String getUSER_DEFINED_IDX12() {
        return USER_DEFINED_IDX12;
    }

    public void setUSER_DEFINED_IDX12(String USER_DEFINED_IDX12) {
        this.USER_DEFINED_IDX12 = USER_DEFINED_IDX12;
    }

    public String getUSER_DEFINED_IDX13() {
        return USER_DEFINED_IDX13;
    }

    public void setUSER_DEFINED_IDX13(String USER_DEFINED_IDX13) {
        this.USER_DEFINED_IDX13 = USER_DEFINED_IDX13;
    }

    public String getUSER_DEFINED_IDX14() {
        return USER_DEFINED_IDX14;
    }

    public void setUSER_DEFINED_IDX14(String USER_DEFINED_IDX14) {
        this.USER_DEFINED_IDX14 = USER_DEFINED_IDX14;
    }

    public String getUSER_DEFINED_IDX15() {
        return USER_DEFINED_IDX15;
    }

    public void setUSER_DEFINED_IDX15(String USER_DEFINED_IDX15) {
        this.USER_DEFINED_IDX15 = USER_DEFINED_IDX15;
    }

    public String getUSER_DEFINED_IDX16() {
        return USER_DEFINED_IDX16;
    }

    public void setUSER_DEFINED_IDX16(String USER_DEFINED_IDX16) {
        this.USER_DEFINED_IDX16 = USER_DEFINED_IDX16;
    }

    public String getUSER_DEFINED_IDX17() {
        return USER_DEFINED_IDX17;
    }

    public void setUSER_DEFINED_IDX17(String USER_DEFINED_IDX17) {
        this.USER_DEFINED_IDX17 = USER_DEFINED_IDX17;
    }

    public String getUSER_DEFINED_IDX18() {
        return USER_DEFINED_IDX18;
    }

    public void setUSER_DEFINED_IDX18(String USER_DEFINED_IDX18) {
        this.USER_DEFINED_IDX18 = USER_DEFINED_IDX18;
    }

    public String getUSER_DEFINED_IDX19() {
        return USER_DEFINED_IDX19;
    }

    public void setUSER_DEFINED_IDX19(String USER_DEFINED_IDX19) {
        this.USER_DEFINED_IDX19 = USER_DEFINED_IDX19;
    }

    public String getUSER_DEFINED_IDX20() {
        return USER_DEFINED_IDX20;
    }

    public void setUSER_DEFINED_IDX20(String USER_DEFINED_IDX20) {
        this.USER_DEFINED_IDX20 = USER_DEFINED_IDX20;
    }

    public List<String> getProjectList() {
        return projectList;
    }

    public void setProjectList(List<String> projectList) {
        this.projectList = projectList;
    }

    public List<String> getDeletedProjectList() {
        return deletedProjectList;
    }

    public void setDeletedProjectList(List<String> deletedProjectList) {
        this.deletedProjectList = deletedProjectList;
    }

    public List<String> getBugIds() {
        return bugIds;
    }

    public void setBugIds(List<String> bugIds) {
        this.bugIds = bugIds;
    }

    public List<String> getBugGrades() {
        return bugGrades;
    }

    public void setBugGrades(List<String> bugGrades) {
        this.bugGrades = bugGrades;
    }

    public List<String> getBugTypes() {
        return bugTypes;
    }

    public void setBugTypes(List<String> bugTypes) {
        this.bugTypes = bugTypes;
    }

    public List<String> getEmergencies() {
        return emergencies;
    }

    public void setEmergencies(List<String> emergencies) {
        this.emergencies = emergencies;
    }

    public List<String> getStates() {
        return states;
    }

    public void setStates(List<String> states) {
        this.states = states;
    }

    public List<String> getCreatedBys() {
        return createdBys;
    }

    public void setCreatedBys(List<String> createdBys) {
        this.createdBys = createdBys;
    }

    public String getCreatedTimeStart() {
        return createdTimeStart;
    }

    public void setCreatedTimeStart(String createdTimeStart) {
        this.createdTimeStart = createdTimeStart;
    }

    public String getCreatedTimeEnd() {
        return createdTimeEnd;
    }

    public void setCreatedTimeEnd(String createdTimeEnd) {
        this.createdTimeEnd = createdTimeEnd;
    }

    public List<String> getReceivers() {
        return receivers;
    }

    public void setReceivers(List<String> receivers) {
        this.receivers = receivers;
    }

    public String getRepairTimeStart() {
        return repairTimeStart;
    }

    public void setRepairTimeStart(String repairTimeStart) {
        this.repairTimeStart = repairTimeStart;
    }

    public String getRepairTimeEnd() {
        return repairTimeEnd;
    }

    public void setRepairTimeEnd(String repairTimeEnd) {
        this.repairTimeEnd = repairTimeEnd;
    }

    public List<String> getRepairUsers() {
        return repairUsers;
    }

    public void setRepairUsers(List<String> repairUsers) {
        this.repairUsers = repairUsers;
    }

    public String getCloseTimeStart() {
        return closeTimeStart;
    }

    public void setCloseTimeStart(String closeTimeStart) {
        this.closeTimeStart = closeTimeStart;
    }

    public String getCloseTimeEnd() {
        return closeTimeEnd;
    }

    public void setCloseTimeEnd(String closeTimeEnd) {
        this.closeTimeEnd = closeTimeEnd;
    }

    @Override
    public String toString() {
        return "BugManagementQueryDto{" +
                "bugGrades=" + bugGrades +
                ", bugTypes=" + bugTypes +
                ", emergencies=" + emergencies +
                ", states=" + states +
                ", createdBys=" + createdBys +
                ", createdTimeStart=" + createdTimeStart +
                ", createdTimeEnd=" + createdTimeEnd +
                ", receivers=" + receivers +
                ", repairTimeStart=" + repairTimeStart +
                ", repairTimeEnd=" + repairTimeEnd +
                ", repairUsers=" + repairUsers +
                ", closeTimeStart=" + closeTimeStart +
                ", closeTimeEnd=" + closeTimeEnd +
                ", bugIds=" + bugIds +
                ", projectList=" + projectList +
                ", USER_DEFINED_IDX1='" + USER_DEFINED_IDX1 + '\'' +
                ", USER_DEFINED_IDX2='" + USER_DEFINED_IDX2 + '\'' +
                ", USER_DEFINED_IDX3='" + USER_DEFINED_IDX3 + '\'' +
                ", USER_DEFINED_IDX4='" + USER_DEFINED_IDX4 + '\'' +
                ", USER_DEFINED_IDX5='" + USER_DEFINED_IDX5 + '\'' +
                ", USER_DEFINED_IDX6='" + USER_DEFINED_IDX6 + '\'' +
                ", USER_DEFINED_IDX7='" + USER_DEFINED_IDX7 + '\'' +
                ", USER_DEFINED_IDX8='" + USER_DEFINED_IDX8 + '\'' +
                ", USER_DEFINED_IDX9='" + USER_DEFINED_IDX9 + '\'' +
                ", USER_DEFINED_IDX10='" + USER_DEFINED_IDX10 + '\'' +
                ", USER_DEFINED_IDX11='" + USER_DEFINED_IDX11 + '\'' +
                ", USER_DEFINED_IDX12='" + USER_DEFINED_IDX12 + '\'' +
                ", USER_DEFINED_IDX13='" + USER_DEFINED_IDX13 + '\'' +
                ", USER_DEFINED_IDX14='" + USER_DEFINED_IDX14 + '\'' +
                ", USER_DEFINED_IDX15='" + USER_DEFINED_IDX15 + '\'' +
                ", USER_DEFINED_IDX16='" + USER_DEFINED_IDX16 + '\'' +
                ", USER_DEFINED_IDX17='" + USER_DEFINED_IDX17 + '\'' +
                ", USER_DEFINED_IDX18='" + USER_DEFINED_IDX18 + '\'' +
                ", USER_DEFINED_IDX19='" + USER_DEFINED_IDX19 + '\'' +
                ", USER_DEFINED_IDX20='" + USER_DEFINED_IDX20 + '\'' +
                ", linkId='" + linkId + '\'' +
                '}';
    }
}