/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.cache.EntityNameCache;
import net.northking.cctp.common.cache.EntityNameCatalog;
import net.northking.cctp.common.db.BasicService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.annotation.DatabaseReadOnly;
import net.northking.cctp.common.db.annotation.DatabaseWriteOnly;
import net.northking.cctp.common.db.excel.AbstractExcelService;
import net.northking.cctp.common.db.excel.ExcelHeader;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.util.UUIDUtil;

import net.northking.cctp.dw.db.service.DwTestPointService;
import net.northking.cctp.dw.db.service.MtTestPointStatisticsService;
import net.northking.cctp.dw.db.entity.MtTestPointStatistics;

import net.northking.cctp.dw.dto.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 需求分析统计图服务实现类
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */

@Service
@ConditionalOnMissingBean(name = "MtTestPointStatisticsApiServiceImpl")
public class MtTestPointStatisticsApiServiceImpl extends AbstractExcelService<MtTestPointStatistics> implements MtTestPointStatisticsApiService
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(MtTestPointStatisticsApiServiceImpl.class);

  @Autowired
  private MtTestPointStatisticsService mtTestPointStatisticsService;
  @Autowired
  private DwTestPointService dwTestPointService;
  @Autowired
  private EntityNameCache entityNameCache;

  //= Excel 导入导出相关代码 start ==================//
  @Override
  public BasicService<MtTestPointStatistics> getService()
  {
    return this.mtTestPointStatisticsService;
  }

  @Override
  public ExcelHeader[] excelHeaders()
  {
    ExcelHeader id = new ExcelHeader("", "id", String.class);
    ExcelHeader userName = new ExcelHeader("测试点创建人", "userName", String.class);
    ExcelHeader loginNo = new ExcelHeader("登陆账号", "loginNo", String.class);
    ExcelHeader testPointCount = new ExcelHeader("测试点数量", "testPointCount", Integer.class);
    ExcelHeader createdTime = new ExcelHeader("创建时间", "createdTime", Date.class);
    ExcelHeader projectId = new ExcelHeader("", "projectId", String.class);
    ExcelHeader tenantId = new ExcelHeader("租户id", "tenantId", String.class);
    // TODO 定义表头及顺序
    return new ExcelHeader[]{};
  }

  /**
   * Excel导入，入库之前的处理
   * @param records 从Excel读取到的记录
   * @return 待入库的记录
   */
  @Override
  protected List<MtTestPointStatistics> beforeInsertBatch(List<MtTestPointStatistics> records)
  {
    for (MtTestPointStatistics record : records)
    {
      record.setId(UUIDUtil.create32UUID());
    }
    return records;
  }

  /**
   * 写入Excel之前的处理<br>
   * 数据的特殊处理在此处加工处理
   *
   * @param record 来自数据库记录
   * @return 待写入Excel的记录
   */
  @Override
  protected MtTestPointStatistics beforeWriteExcel(MtTestPointStatistics record)
  {
    return record;
  }

  //= Excel 导入导出相关代码 end =======================//

  //= 增删改查 相关代码 start ==================//
  @Override
  @DatabaseReadOnly
  public Pagination<MtTestPointStatisticsDetailDto> pagingQuery(QueryByPage<MtTestPointStatisticsQueryDto> queryByPage)
  {
    return this.mtTestPointStatisticsService.pagingQuery(queryByPage);
  }

  @Override
  @DatabaseWriteOnly
  public String add(MtTestPointStatisticsAddDto dto)
  {
    String uuid = UUIDUtil.create32UUID();
    MtTestPointStatistics entity = new MtTestPointStatistics();
    BeanUtils.copyProperties(dto, entity);
    entity.setId(uuid);
    this.mtTestPointStatisticsService.insert(entity);
    return uuid;
  }

  @Override
  @DatabaseWriteOnly
  public Integer updateByPK(MtTestPointStatisticsUpdateDto dto) throws IllegalAccessException
  {
    MtTestPointStatistics entity = new MtTestPointStatistics();
    BeanUtils.copyProperties(dto, entity);
    return this.mtTestPointStatisticsService.updateByPrimaryKey(entity);
  }

  @Override
  @DatabaseReadOnly
  public MtTestPointStatisticsDetailDto findByPK(String uuid)
  {
    MtTestPointStatistics example = new MtTestPointStatistics();
    example.setId(uuid);
    List<MtTestPointStatistics> entitys = this.mtTestPointStatisticsService.query(example);
    if (entitys.isEmpty())
    {
      return null;
    }

    MtTestPointStatisticsDetailDto dto = new MtTestPointStatisticsDetailDto();
    BeanUtils.copyProperties(entitys.get(0), dto);
    return dto;
  }

  @Override
  @DatabaseWriteOnly
  public Integer deleteByPK(String uuid)
  {
    MtTestPointStatistics entity = new MtTestPointStatistics();
    entity.setId(uuid);
    return this.mtTestPointStatisticsService.deleteByExample(entity);
  }

  @Override
  public List<String> addBatch(List<MtTestPointStatisticsAddDto> batch)
  {
    List<String> result = new ArrayList<>();
    for (MtTestPointStatisticsAddDto dto : batch)
    {
      result.add(this.add(dto));
    }
    return result;
  }

  @Override
  public Integer updateBatch(List<MtTestPointStatisticsUpdateDto> batch) throws IllegalAccessException
  {
    int sum = 0;
    for (MtTestPointStatisticsUpdateDto dto : batch)
    {
      sum = sum + this.updateByPK(dto);
    }
    return sum;
  }

  @Override
  public Integer deleteBatch(List<Object> batch)
  {
    return this.mtTestPointStatisticsService.deleteByPrimaryKeys(batch);
  }

  @Override
  public List<MtTestPointStatisticsDetailDto> listQuery(ChartDataGeneQueryDto chartDataGeneQueryDto) {
    List<MtTestPointStatisticsDetailDto> result = dwTestPointService.findTestPointStatistics(chartDataGeneQueryDto);
    for (MtTestPointStatisticsDetailDto dto : result) {
      dto.setUserName(entityNameCache.translateById(EntityNameCatalog.WITH_LOGIN_NO, dto.getUserName()));
    }
    return result;
  }

}
