/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.saas.api.tenant;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.saas.api.tenant.service.SaaSTenantApiService;
import net.northking.cctp.saas.dto.tenant.SaaSTenantDetailDto;
import net.northking.cctp.saas.dto.tenant.SaaSTenantQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "租户")
@RequestMapping("/v1")
@RestController
public class SaaSTenantQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(SaaSTenantQueryCtrl.class);

  private SaaSTenantApiService apiService;

  @Autowired
  public void setSaasTenantRestfulService(SaaSTenantApiService apiService)
  {
    this.apiService = apiService;
  }

  /**
   * 查询用户列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表")
  @PostMapping(value = "/paging/query/tenants",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<SaaSTenantDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<SaaSTenantQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<SaaSTenantDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<SaaSTenantDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }
}


